source core.tcl
source random.tcl

set tw 80
set th 60
set bdl 120
set bdt 80
set bdr 50
set bdb 50
global tw th bdl bdt bdr bdb 

array set strokes {
    ne {202242}  se {242242} sw {242202} nw {202202} 
    ho {0242} ve {2024} cr {2024 0242} bl {} 
    xne {202242} xse {242242} xsw {242202} xnw {202202}   
    xho {0242} xve {2024} xx {}
    stn {212220}  ste {322242}  sts {232224}  stw {122202}
    xstn {212220}  xste {322242}  xsts {232224}  xstw {122202}
    enn {212220}  ene {322242}  ens {232224}  enw {122202}
    xenn {212220}  xene {322242}  xens {342224}  xenw {122202}
    boh {0242} bov {2024} xboh {0242} xbov {2024} 
}

proc xsetblock {x y} { global xy; drawface $x $y $xy($x$y) }
proc drawface {x y t} { global tw th bdl bdt bdr bdb strokes
    set tag t$x$y; .c delete withtag $tag
    switch -glob $t {
        x* - cr {set col "#c0c000"}
        st* {set col "#c00000"}
        en* {set col "#d00000"}
        bo* {set col "#00c000"}
        bl {set col black}
        default {set col "#804000"}
    }
    .c create rect 0 0 4 4 -width 1 -fill $col -outline black -tags $tag
    foreach stroke $strokes($t) { 
        set id [ .c create line 0 0 0 0 -width 20 -fill "#008080" -tags $tag ]
        eval .c coords $id [split $stroke {}]
    }
    .c scale $tag 0 0 [* $tw 0.25] [* $th 0.25]
    .c move $tag [+ [* $x $tw] $bdl] [+ [* $y $th] $bdt]
    .c raise ant
}

proc antpos {} { global antx anty antst antd1 antd2 hsPt sPt
    global tw th bdl bdt bdr bdb 
    set x [* 0.5 $tw]; set y [* 0.5 $th]
    if {$antst<$hsPt} {
        switch -- $antd1 {
            n { set y [expr {$th-($th * $antst)/$sPt}] }
            s { set y [expr {($th * $antst)/$sPt}] }
            w { set x [expr {$tw-($tw * $antst)/$sPt}] }
            e { set x [expr {($tw * $antst)/$sPt}] }
        }
    } else { set st [expr {$sPt - $antst}]
        switch -- $antd2 {
            n { set y [expr {($th * $st)/$sPt}] }
            s { set y [expr {$th-($th * $st)/$sPt}] }
            w { set x [expr {($tw * $st)/$sPt}] }
            e { set x [expr {$tw-($tw * $st)/$sPt}] }
        }
    }
    set x [expr {int($tw*$antx+$x+$bdl)}]
    set y [expr {int($th*$anty+$y+$bdt)}]
    list $x $y
}

proc xupdate_ant   {} {
    global antx anty antd antst alive fast
    global tw th bdl bdt bdr bdb 
    if {$antst >=0} {
        if {$fast && $alive} {
            eval .c coords ant [antpos]
            #.c coords ant -100 -100
        } else {
            eval .c coords ant [antpos]
        }
    }   else {
        set x [expr {$bdl+$tw*4 + $antst*20}]
        .c coords ant $x 20
    }
    switch -- $antd {
        n - s {.c itemconf antbody -text "8"}
        e - w {.c itemconf antbody -text "oo"}
        default {.c itemconf antbody -text "&"}
    }
    .c raise ant
    update idletasks
}

proc xdeadend      {} {
    xupdate_ant
}

proc xmileage {} {uplevel "#0" {set inches $antma}}

proc click {x y {ch 0}} { global tw th bdl bdt
    set xc [expr {int (($x - $bdl) / $tw)}]
    set yc [expr {int (($y - $bdt) / $th)}]
    if {$xc>=0 && $xc <8 && $yc >=0 && $yc < 6} {rotate $xc $yc $ch} \
    else {}
}

canvas .c -width [expr {$bdl+8*$tw+$bdr}] -height [expr {$bdt+6*$th+$bdb}]
pack .c -expand yes -fill both

.c create text 0 0 -text "oo" -tags {ant antbody} -anchor center
.c bind ant <1> pausemode

frame .c.f 
.c create window 30 100 -window .c.f -anchor nw
button .c.f.quit -text "Quit" -underline 0 -command {exit}
    bind . <q> {.c.f.quit invoke}
button .c.f.new -text "New Game" -underline 0 -command new_game
    bind . <n> {.c.f.new invoke}
button .c.f.pause -text "Pause" -underline 0 -command pausemode
    bind . <p> {.c.f.pause invoke}
button .c.f.fast -text "Fast" -underline 0 -command fastmode
    bind . <f> {.c.f.fast invoke}
button .c.f.help -text "Help" -underline 0 -command #showhelp
    bind . <h> {.c.f.help invoke}
button .c.f.random -text "Random" -underline 0 -command display_random
    bind . <r> {.c.f.random invoke}
checkbutton .c.f.cheat -text "Cheat" -indicatoron 0 -variable cheat 
    bind . <C> {.c.f.cheat invoke}
pack .c.f.quit .c.f.new .c.f.pause .c.f.fast .c.f.help .c.f.random \
    .c.f.cheat -side top -expand 1 -fill both

bind .c <1> {click %x %y}
foreach b {<3> <Shift-1>} {bind .c $b {click %x %y 1}}

scale .c.miles -variable inches -takefocus 0 -from 20 -to 0 \
    -showvalue 1 -state disabled -tickinterval 0 -sliderlength 5 \
    -length 200 
.c create window [winfo reqwidth .c] [winfo reqheight .c] \
    -window .c.miles -anchor se

save_random
initboard 1
step
