# user-configurable
set msecsPstep   100 ;#  milliseconds between single steps
set stepsPerTile 26  ;#  how many steps are needed for a tile

# constant  scalars & lists
set hsPt  [expr {int($stepsPerTile/2)}]; set sPt [expr {$hsPt*2}]
set xl {0 1 2 3 4 5 6 7}; set yl {0 1 2 3 4 5}
set dirs {n e s w}; array set opposite {s n  n s  e w  w e}
set tiles {
    ne  se  sw  nw   ho  ve   cr 
    xne xse xsw xnw  xho xve  xx  bl  
    stn  ste  sts  stw  enn  ene  ens  enw 
    xstn xste xsts xstw xenn xene xens xenw 
    boh bov xboh xbov
}

# constant tables
array set enter_w { ne n  se s  ho w  cr w  boh w  ene f  ste x}
array set enter_e { nw n  sw s  ho e  cr e  boh e  enw f  stw x}
array set enter_s { ne e  nw w  ve s  cr s  bov s  enn f  stn x}
array set enter_n { se e  sw w  ve n  cr n  bov n  ens f  sts x}
array set rot {
    ne se     se sw     sw nw     nw ne
    stn ste   ste sts   sts stw   stw stn
    enn ene   ene ens   ens enw   enw enn
    ho ve     ve ho     boh bov   bov boh
}

# dummy "prototypes"
proc xsetblock     {x y} {}; # override for GUI
proc xupdate_ant   {}    {}; # override for GUI
proc xdeadend      {}    {}; # override for GUI
proc xinitboard    lvl   {}; # override for GUI
proc xmileage      {}    {}; # override for GUI


# procedures
proc * {x y} {expr {$x * $y}}
proc + {x y} {expr {$x + $y}}
proc initboard {lvl} { global xl yl dirs tiles starts 
    global   lasttile alive fast pause cheat cheatcnt
    foreach {lasttile alive fast pause cheat cheatcnt} \
            {    {}     0     0    0     0       0   } {}
    global   antx anty antst antd1 antd2 antd antma antclr
    foreach {antx anty antst antd1 antd2 antd antma antclr} \
            { -1   -1   -10    x     x     e    0      0  } {}
    foreach x $xl {
        foreach y $yl {
            setblock $x$y [lindex $tiles [random 7]]
        }
    }
    while {[random 3]>1} {
        set x [random 8]; set y [random 6]
        setblock $x$y  bo[lindex {h v} [random 2]]
    }
    set x [random 8]; set y [random 6]
    set starts $x$y
    setblock $x$y  st[lindex $dirs [random 4]]
    xinitboard $lvl; xmileage
}
proc resetboard {} { global xl yl dirs tiles starts lasttile xy
    global antclr; set antclr 0
    set list {}
    foreach x $xl {
        foreach y $yl { set t $xy($x$y)
            if {[string match x* $t]} {
                lappend list $x$y
            }
        }
    }
    set lasttile {}
    foreach t $list {
        setblock $t [lindex $tiles [random 7]]
    }
    # one of them becomes blank
    setblock [lindex $list [random [llength $list]]] bl
}

proc move {} { global antx  anty  antd1  antd2  antst antma
    global fast antclr antwrap 
    set xn $antx; set yn $anty; set antwrap 0
    switch -exact -- $antd2 {
    n { incr yn -1; if {$yn < 0} {set yn 5; set antwrap 1} }
    e { incr xn   ; if {$xn > 7} {set xn 0; set antwrap 1} }
    s { incr yn   ; if {$yn > 5} {set yn 0; set antwrap 1} }
    w { incr xn -1; if {$xn < 0} {set xn 7; set antwrap 1} }
    }

    if {$antwrap && $fast} {
        set fast 0
    }
    set dn [check $antd2 $xn $yn]
    if {[eq $dn {}]} { # endstation
        return 0
    }
    global  lasttile; set lasttile $antx$anty

    incr antma
    if {$antma==20} {set antclr 1}
    if {$antma==21} {set antma 1}
    xmileage
    if {$antclr && $antwrap} {resetboard}

    upvar "#0" xy($xn$yn) t
    if {![eq $t "cr"]} { setblock $xn$yn x$t }

    set antx $xn; set anty $yn
    set antd1 $antd2; set antd2 $dn 
    set antst 0; return 1
}

proc step {} {
    global antx  anty  antd  antd1  antd2  antst
    global   msecsPstep  sPt  hsPt
    global  lasttile xy starts alive pause fast

    set pause 0; # if we arrive here, user chose to continue
    if {$alive} {
        if {$fast} {
            if {$antst<0} {incr antst} \
            elseif {$antst<$hsPt} {set antst $hsPt} \
            elseif {$antst<$sPt} {set antst $sPt} \
            else {set antst $sPt}
        } else {incr antst}
    }

    if {$antst < 0} {
        set alive 1
        xupdate_ant
        after [expr {$fast ? 10 : 1000}] step
        return
    } elseif {$antst == 0} {
        set s [lindex $starts [random [llength $starts]]]
        set antst $hsPt; set fast 0
        foreach {antx anty} [split $s {}] {}
        set antd2 [string index $xy($s) 2]; set antd $antd2
        foreach s $starts { setblock $s x$xy($s) }
    } elseif {$antst == $hsPt} {
        if {![eq $lasttile {}] && [string match x* $xy($lasttile)]
        } then {setblock $lasttile xx}
        set antd $antd2
        if {[string match {[fx]} $antd]} {deadend; return}
    } elseif {$antst == $sPt} {
        if [move] {set antd $antd1} else {set antd x; deadend; return}
    }

    set alive 1; xupdate_ant
    after [expr {$fast ? 50 : $msecsPstep}] step
}

proc deadend {} { global alive fast 
    set alive 0; set fast 0; xdeadend; return
}
proc pausemode {} { global pause alive
    if {!$alive || $pause} {step} \
    else {set pause 1; after cancel step}
}
proc fastmode {} { global fast alive 
    if {$alive} { after cancel step; set fast 1; step }
}
proc new_game {} {initboard 1; step}


proc check {d x y} { global dir
    upvar "#0" xy($x$y) xy
    upvar "#0" enter_$d  gate
    if {[info exists gate($xy)]} {
        return $gate($xy)
    } else { return {} }
}

proc setblock {pxy p} { 
    uplevel "#0" [list set xy($pxy) $p]
    eval xsetblock [split $pxy {}]
}

proc rotate {x y {ch 0}} { global rot cheat
    upvar "#0" xy($x$y) xy
    if {$ch && $cheat} { global cheatcnt; incr cheatcnt
        setblock $x$y [lindex {cr nw ve} [expr {$cheatcnt % 3}]]
    } elseif {[info exist rot($xy)]} {
        setblock $x$y $rot($xy)
    }
}        

# in some later tcl-version, this will be obsolete, but now ...
proc eq {s1 s2} {expr {"S$s1" == "S$s2"}}


